@@ -190,8 +190,8 @@ gnome_entry_class_init (GnomeEntryClass *class)
 	g_object_class_install_property (gobject_class,
 					 PROP_HISTORY_ID,
 					 g_param_spec_string ("history_id",
-							      _("History id"),
-							      _("History id"),
+							      _("History ID"),
+							      _("History ID"),
 							      NULL,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_class,
@@ -324,9 +324,7 @@ free_item (gpointer data, gpointer user_data)
 	struct item *item;
 
 	item = data;
-	if (item->text)
-		g_free (item->text);
-
+	g_free (item->text);
 	g_free (item);
 }
 
