@@ -30,8 +30,11 @@
  * Authors: Federico Mena <federico@nuclecu.unam.mx>
  */
 #include <config.h>
+#include <glib.h>
 #include <sys/param.h>
+#ifndef G_OS_WIN32
 #include <pwd.h>
+#endif
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -103,6 +106,10 @@ static void fentry_get_property (GObject *object, guint param_id,
 
 static void gnome_file_entry_editable_init (GtkEditableClass *iface);
 
+#ifdef G_OS_WIN32
+#define realpath(a, b) strcpy (b, a)
+#endif
+
 /* Property IDs */
 enum {
 	PROP_0,
@@ -305,13 +312,12 @@ gnome_file_entry_class_init (GnomeFileEntryClass *class)
 						 FALSE,
 						 G_PARAM_READWRITE));
 
-	/* FIXME: mark the property strings for translation in 2.7.x */
 	g_object_class_install_property (gobject_class,
 					 PROP_FILECHOOSER_ACTION,
 					 g_param_spec_enum (
 						 "filechooser_action",
-						 "GtkFileChooser Action",
-						 "The type of operation that the file selector is performing",
+						 _("GtkFileChooser Action"),
+						 _("The type of operation that the file selector is performing."),
 						 GTK_TYPE_FILE_CHOOSER_ACTION,
 						 GTK_FILE_CHOOSER_ACTION_OPEN,
 						 G_PARAM_READWRITE));
@@ -439,7 +445,7 @@ browse_dialog_ok (GtkWidget *widget, gpointer data)
 	GtkWidget *fw;
 	GnomeFileEntry *fentry;
 	GtkWidget *entry;
-	const gchar *locale_filename;
+	gchar *locale_filename;
 	gchar *utf8_filename;
 
 	fw = GTK_WIDGET (data);
@@ -449,10 +455,11 @@ browse_dialog_ok (GtkWidget *widget, gpointer data)
 	if (GTK_IS_FILE_CHOOSER (fentry->fsw))
 		locale_filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fw));
 	else
-		locale_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fw));
+		locale_filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (fw)));
 
 	utf8_filename = g_filename_to_utf8 (locale_filename, -1, NULL,
 					    NULL, NULL);
+	g_free (locale_filename);
 	gtk_entry_set_text (GTK_ENTRY (entry), utf8_filename);
 	g_free (utf8_filename);
 	/* Is this evil? */
@@ -478,6 +485,16 @@ browse_dialog_kill (GtkWidget *widget, gpointer data)
 	fentry->fsw = NULL;
 }
 
+static void
+setup_filter(GtkFileChooser *fc)
+{
+	GtkFileFilter *filter;
+	filter = gtk_file_filter_new();
+	gtk_file_filter_add_mime_type(filter, "x-directory/normal");
+	gtk_file_chooser_set_filter(fc, filter);
+}
+
+#ifndef G_OS_WIN32
 /* Does tilde (home directory) expansion on a string */
 static char *
 tilde_expand (const char *str)
@@ -516,6 +533,7 @@ tilde_expand (const char *str)
 
 	return g_strconcat (passwd->pw_dir, G_DIR_SEPARATOR_S, p, NULL);
 }
+#endif
 
 /* This is only used for the file dialog.  It can end up also being
  * the default path. */
@@ -523,6 +541,7 @@ static gchar *
 build_filename (GnomeFileEntry *fentry)
 {
 	const char *text;
+	char *locale_text;
 	char *file;
 	int len;
 
@@ -533,11 +552,17 @@ build_filename (GnomeFileEntry *fentry)
 		(GTK_ENTRY (gnome_file_entry_gtk_entry (fentry)));
 
 	if (text == NULL || text[0] == '\0')
-		return g_strconcat (fentry->default_path, "/", NULL);
+		return g_strconcat (fentry->default_path, G_DIR_SEPARATOR_S, NULL);
 
-	file = _gnome_file_entry_expand_filename (text, fentry->default_path);
+	locale_text = g_filename_from_utf8 (text, -1, NULL, NULL, NULL);
+	if (locale_text == NULL)
+		return g_strconcat (fentry->default_path, G_DIR_SEPARATOR_S, NULL);	
+	
+	file = _gnome_file_entry_expand_filename (locale_text, fentry->default_path);
+	g_free (locale_text);
+	
 	if (file == NULL)
-		return g_strconcat (fentry->default_path, "/", NULL);
+		return g_strconcat (fentry->default_path, G_DIR_SEPARATOR_S, NULL);
 
 	/* Now append a '/' if it doesn't exist and we're in directory only
 	 * mode.  We also have to do this if the file exists and it *is* a
@@ -549,7 +574,7 @@ build_filename (GnomeFileEntry *fentry)
 	len = strlen (file);
 
 	if (len != 0
-	    && file[len - 1] != G_DIR_SEPARATOR
+	    && !G_IS_DIR_SEPARATOR (file[len - 1])
 	    && (fentry->_priv->directory_entry || g_file_test (file, G_FILE_TEST_IS_DIR))) {
 		gchar *tmp;
 
@@ -639,6 +664,9 @@ browse_clicked(GnomeFileEntry *fentry)
 					       GTK_RESPONSE_ACCEPT);
 
 		fw = fentry->fsw;
+		if (fentry->_priv->directory_entry)
+			g_signal_connect(fw, "size-request",
+				G_CALLBACK(setup_filter), fentry);
 
 		gtk_dialog_set_default_response (GTK_DIALOG (fw), GTK_RESPONSE_ACCEPT);
 
@@ -733,7 +761,7 @@ gnome_file_entry_drag_data_received (GtkWidget        *widget,
 
 	entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (widget));
 
-	uris = g_strsplit (selection_data->data, "\r\n", -1);
+	uris = g_strsplit ((char *)selection_data->data, "\r\n", -1);
 	if (uris == NULL) {
 		gtk_drag_finish (context, FALSE, FALSE, time);
 		return;
@@ -956,9 +984,7 @@ gnome_file_entry_set_title (GnomeFileEntry *fentry, const char *browse_dialog_ti
 	g_return_if_fail (fentry != NULL);
 	g_return_if_fail (GNOME_IS_FILE_ENTRY (fentry));
 
-	if (fentry->_priv->browse_dialog_title)
-		g_free (fentry->_priv->browse_dialog_title);
-
+	g_free (fentry->_priv->browse_dialog_title);
 	fentry->_priv->browse_dialog_title = g_strdup (browse_dialog_title); /* handles NULL correctly */
 }
 
@@ -975,12 +1001,21 @@ gnome_file_entry_set_title (GnomeFileEntry *fentry, const char *browse_dialog_ti
 void
 gnome_file_entry_set_default_path(GnomeFileEntry *fentry, const char *path)
 {
+#ifdef MAXPATHLEN
 	char rpath[MAXPATHLEN+1];
+#else
+	char *rpath;
+#endif
 	char *p;
 	g_return_if_fail (fentry != NULL);
 	g_return_if_fail (GNOME_IS_FILE_ENTRY (fentry));
 
+#ifdef MAXPATHLEN
 	if(path && realpath(path, rpath))
+#else
+	rpath = realpath(path, NULL);
+	if(path && rpath)
+#endif
 		p = g_strdup(rpath);
 	else
 		p = NULL;
@@ -998,17 +1033,21 @@ _gnome_file_entry_expand_filename (const char *input, const char *default_dir)
 		return NULL;
 	} else if (input[0] == '\0') {
 		return g_strdup ("");
-	} else if (input[0] == G_DIR_SEPARATOR) {
+	} else if (g_path_is_absolute (input)) {
 		return g_strdup (input);
+#ifndef G_OS_WIN32
 	} else if (input[0] == '~') {
 		return tilde_expand (input);
+#endif
 	} else if (default_dir != NULL) {
 		char *ret = g_build_filename (default_dir, input, NULL);
+#ifndef G_OS_WIN32
 		if (ret[0] == '~') {
 			char *tmp = tilde_expand (ret);
 			g_free (ret);
 			return tmp;
 		}
+#endif
 		return ret;
 	} else {
 		char *cwd = g_get_current_dir ();
@@ -1038,6 +1077,7 @@ char *
 gnome_file_entry_get_full_path(GnomeFileEntry *fentry, gboolean file_must_exist)
 {
 	const char *text;
+	char *sys_text;
 	char *file;
 
 	g_return_val_if_fail (fentry != NULL, NULL);
@@ -1049,7 +1089,13 @@ gnome_file_entry_get_full_path(GnomeFileEntry *fentry, gboolean file_must_exist)
 	if (text == NULL || text[0] == '\0')
 		return NULL;
 
-	file = _gnome_file_entry_expand_filename (text, fentry->default_path);
+	sys_text = g_filename_from_utf8 (text, -1, NULL, NULL, NULL);
+	if (sys_text == NULL)
+		return NULL;
+	
+	file = _gnome_file_entry_expand_filename (sys_text, fentry->default_path);
+	g_free (sys_text);
+	
 	if (file == NULL)
 		return NULL;
 
